#!/usr/bin/python

import sys
import re
import datetime
from operator import itemgetter, attrgetter
from optparse import OptionParser

parser = OptionParser()
parser.add_option("-r", "--regular", action='store_true')
parser.add_option("-l", "--linaro", action='store_true')
parser.add_option("-c", "--canonical", action='store_true')
parser.add_option("-w", "--wiki", action='store_true')

(options, args) = parser.parse_args()

filename = args[0]

row_re = re.compile(r"<table:table-row.*?>(.*?)</table:table-row>")
#cell_re = re.compile(r"<table:table-cell(?: table:number-columns-repeated=\"(?P<repeat>[0-9]+)\")?.*?(?: office:value=\"(?P<value>[0-9\.]+)\")?.*?>(?:<text.*?>)?(?P<text>.*?)(?:</text.*?>)?</table:table-cell>")
cell_re = re.compile(r"<table:table-cell(?P<options> .*?)?>(?:<text.*?>)?(?P<text>.*?)(?:</text.*?>)?</table:table-cell>")
count_re = re.compile(r"\btable:number-columns-repeated=\"(?P<repeat>[0-9]+)\"")
value_re = re.compile(r"\boffice:value=\"(?P<value>.*?)\"")

# <table:table-cell table:style-name="ce23" office:value-type="float" office:value="40846.0"><text:p>10/29/2011</text:p></table:table-cell>

duplicates = 0

rows = {}

titles = None
skip = False
with open(filename) as fd:
    for line in fd:
        for row_m in row_re.finditer(line):
            row = list()
            for cell_m in cell_re.finditer(row_m.group(1)):
                if cell_m.group('options'):
                    count_m = count_re.search(cell_m.group('options'))
                    if count_m:
                        count = int(count_m.group('repeat'))
                    else:
                        count = 1
                text = cell_m.group('text')
                if cell_m.group('options'):
                    value_m = value_re.search(cell_m.group('options'))
                if value_m:
                    value = float(value_m.group('value'))
                    if len(row) > 3 and value > 40000 and value < 50000:
                        text = str(datetime.date(1899, 12, 30) + datetime.timedelta(value))

                for i in range(count):
                    row.append(text)

            if len(row) < 5 or row[0] == '':
                continue;
            if row[0] == 'Submitted Date':
                if options.linaro and row[10] == 'Linaro Engineering Team:':
                    skip = False
                elif (options.regular or options.canonical) and (row[9] != 'Linaro Engineering Team:' and row[10] != 'Linaro Engineering Team:'):
                    skip = False
                else:
                    skip = True
                if not skip and not titles:
                    titles = row[0:30]
                continue

            if skip:
                continue

            # Truncate the row to something vaguly sensible.
            row = row[0:30]

            # Email address is the key for duplicate detection
            user = row[5]

            # Extract the year: 8/14/2011 6:46:18
            time_new = datetime.datetime.strptime(row[0], '%m/%d/%Y %H:%M:%S')
            row.insert(0, time_new)

            # If this is a new unique row shove it in.
            if not user in rows:
                rows[user] = list(row)
                continue

            time_old = rows[user][0]

            if time_new > time_old:
                #sys.stderr.write("DUPLICATE %s\n" % (user))
                rows[user] = list(row)
                duplicates += 1


def fix_date(date):
    date = re.sub(r'^(\d\d)/(\d\d)/(\d\d\d\d)$', r'\3-\1-\2', date)
    date = re.sub(r'^(\d\d)/(\d\d)$', r'2011-\1-\2', date)
    date = re.sub(r'^(\d\d\d\d)\.(\d\d)\.(\d\d)$', r'\1-\2-\3', date)
    date = re.sub(r'^(\d\d\d\d)(\d\d)(\d\d)$', r'\1-\2-\3', date)

    return date


def fix_time(time):
    time = re.sub(r'^(\d:)', r'0\1', time)
    time = re.sub(r'^(\d\d:\d\d):00', r'\1', time)

    return time
    

def fix_airline_code(match):
    return match.group(1).upper() + " " + match.group(2)

def fix_airline(airline):
    airline = re.sub(r' *Flight Number:', r'', airline)
    airline = re.sub(r'(Air Lines|Airlines|Airways) *', r'', airline)
    airline = re.sub(r'United *', r'UA', airline)
    airline = re.sub(r'Delta *', r'DL', airline)
    airline = re.sub(r'JetBlue *', r'B6 ', airline)
    airline = re.sub(r'Continental *', r'CO ', airline)

    airline = re.sub(r'^ *([A-Za-z][A-Za-z]|[Bb]6) *(\d\d\d?\d?) *$', fix_airline_code, airline)

    return airline


rows_order = list()
for row in rows:
    rows_order.append(rows[row])

for row in rows_order:
    if options.regular or options.canonical:
        # Fix up the dates.
        row[13] = fix_date(row[13])
        row[16] = fix_date(row[16])

        row[13] = re.sub(r'^Saturday Oct 29th$', r'2011-10-29', row[13])
        row[16] = re.sub(r'^Saturday Nov 5th$', r'2011-11-05', row[16])

        # Fix up the times.
        row[14] = fix_time(row[14])
        row[17] = fix_time(row[17])

        # Fix up the airlines.
        row[15] = fix_airline(row[15])
        row[18] = fix_airline(row[18])

if options.linaro or options.regular:
    rows_order = sorted(rows_order, key=itemgetter(0)) # submission order

    print '\t'.join(titles[3:])
    for row in rows_order:
        print '\t'.join(row[4:])
else:
    wiki = options.wiki
    if wiki:
        print "|| Name || Arrival Date || Arrival Time || Flight || Departure Date || Departure Time || Flight || Roomie ||"
        #rows_order = sorted(rows_order, key=itemgetter(4,5))
        rows_order = sorted(rows_order, key=itemgetter(13,14,4,5)) # arrival time order
    else:
        print "Name\tArrival Date\tArrival Time\tFlight\tDeparture Date\tDeparture Time\tFlight\tRoomie"
        #rows_order = sorted(rows_order, key=itemgetter(0)) # submission order
        #rows_order = sorted(rows_order, key=itemgetter(4,5)) # name order
        rows_order = sorted(rows_order, key=itemgetter(13,14,4,5)) # arrival time order
    for row in rows_order:
        if options.canonical and row[11] != 'Canonical employee':
            continue

        # Fix up the dates.
        row[13] = fix_date(row[13])
        row[16] = fix_date(row[16])

        row[13] = re.sub(r'^Saturday Oct 29th$', r'2011-10-29', row[13])
        row[16] = re.sub(r'^Saturday Nov 5th$', r'2011-11-05', row[16])

        # Fix up the times.
        row[14] = re.sub(r'^(\d:)', r'0\1', row[14])
        row[17] = re.sub(r'^(\d:)', r'0\1', row[17])

        # Fix up the airlines.
        row[15] = fix_airline(row[15])
        row[18] = fix_airline(row[18])

        if wiki:
            print "|| %s %s || %s || %s || %s || %s || %s || %s || %s ||" % (row[4], row[5], row[13], row[14], row[15], row[16], row[17], row[18], row[30])
        else:
            print "%s %s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (row[4], row[5], row[13], row[14], row[15], row[16], row[17], row[18], row[30])

if options.linaro or options.regular:
    sys.stderr.write("DUPLICATES: %d\n" % (duplicates))
